@@ -437,15 +437,18 @@ switch_to_and_unlock_session (GdmSimpleSlave  *slave,
                               gboolean         fail_if_already_switched)
 {
         char    *username;
+        char    *session_id;
         gboolean res;
 
         username = gdm_session_get_username (slave->priv->session);
+        session_id = gdm_session_get_session_id (slave->priv->session);
 
         g_debug ("GdmSimpleSlave: trying to switch to session for user %s", username);
 
         /* try to switch to an existing session */
-        res = gdm_slave_switch_to_user_session (GDM_SLAVE (slave), username, fail_if_already_switched);
+        res = gdm_slave_switch_to_user_session (GDM_SLAVE (slave), username, session_id, fail_if_already_switched);
         g_free (username);
+        g_free (session_id);
 
         return res;
 }
@@ -843,17 +846,7 @@ on_session_client_disconnected (GdmSession          *session,
                                 GPid                 pid_of_client,
                                 GdmSimpleSlave      *slave)
 {
-        gboolean display_is_local;
-
         g_debug ("GdmSimpleSlave: client disconnected");
-
-        g_object_get (slave,
-                      "display-is-local", &display_is_local,
-                      NULL);
-
-        if ( ! display_is_local && !slave->priv->session_is_running) {
-                gdm_slave_stop (GDM_SLAVE (slave));
-        }
 }
 
 static void
@@ -1312,6 +1305,17 @@ static gboolean
 wants_initial_setup (GdmSimpleSlave *slave)
 {
         gboolean enabled = FALSE;
+        gboolean display_is_local = FALSE;
+
+        g_object_get (G_OBJECT (slave),
+                      "display-is-local", &display_is_local,
+                      NULL);
+
+        /* don't run initial-setup on remote displays
+         */
+        if (!display_is_local) {
+                return FALSE;
+        }
 
         /* don't run if the system has existing users */
         if (slave->priv->have_existing_user_accounts) {
